@@ -28,7 +28,7 @@ public:
     size_t string_length() override;
     int value_count(long*) override;
     void destroy(grib_context*) override;
-    void dump(grib_dumper*) override;
+    void dump(eccodes::Dumper*) override;
     void init(const long, grib_arguments*) override;
     int compare(grib_accessor*) override;
 };
