@@ -41,7 +41,7 @@ static pthread_once_t once = PTHREAD_ONCE_INIT;
 static pthread_mutex_t mutex_mem = PTHREAD_MUTEX_INITIALIZER;
 static pthread_mutex_t mutex_c   = PTHREAD_MUTEX_INITIALIZER;
 
-static void init()
+static void init_mutex()
 {
     pthread_mutexattr_t attr;
     pthread_mutexattr_init(&attr);
@@ -55,7 +55,7 @@ static int once = 0;
 static omp_nest_lock_t mutex_mem;
 static omp_nest_lock_t mutex_c;
 
-static void init()
+static void init_mutex()
 {
     GRIB_OMP_CRITICAL(lock_grib_context_c)
     {
@@ -185,19 +185,19 @@ int grib_context_seek(const grib_context* c, off_t offset, int whence, void* str
     return c->seek(c, offset, whence, stream);
 }
 
-int grib_context_eof(const grib_context* c, void* stream)
-{
-    if (!c)
-        c = grib_context_get_default();
-    return c->eof(c, stream);
-}
+// int grib_context_eof(const grib_context* c, void* stream)
+// {
+//     if (!c)
+//         c = grib_context_get_default();
+//     return c->eof(c, stream);
+// }
 
-size_t grib_context_write(const grib_context* c, const void* ptr, size_t size, void* stream)
-{
-    if (!c)
-        c = grib_context_get_default();
-    return c->write(c, ptr, size, stream);
-}
+// size_t grib_context_write(const grib_context* c, const void* ptr, size_t size, void* stream)
+// {
+//     if (!c)
+//         c = grib_context_get_default();
+//     return c->write(c, ptr, size, stream);
+// }
 
 static void default_log(const grib_context* c, int level, const char* mess)
 {
@@ -205,7 +205,7 @@ static void default_log(const grib_context* c, int level, const char* mess)
         c = grib_context_get_default();
     if (level == GRIB_LOG_ERROR) {
         fprintf(c->log_stream, "ECCODES ERROR   :  %s\n", mess);
-        /*Assert(1==0);*/
+        /*ECCODES_ASSERT(1==0);*/
     }
     if (level == GRIB_LOG_FATAL)
         fprintf(c->log_stream, "ECCODES ERROR   :  %s\n", mess);
@@ -217,15 +217,15 @@ static void default_log(const grib_context* c, int level, const char* mess)
         fprintf(c->log_stream, "ECCODES INFO    :  %s\n", mess);
 
     if (level == GRIB_LOG_FATAL) {
-        Assert(0);
+        ECCODES_ASSERT(0);
     }
 
     if (getenv("ECCODES_FAIL_IF_LOG_MESSAGE")) {
         long n = atol(getenv("ECCODES_FAIL_IF_LOG_MESSAGE"));
         if (n >= 1 && level == GRIB_LOG_ERROR)
-            Assert(0);
+            ECCODES_ASSERT(0);
         if (n >= 2 && level == GRIB_LOG_WARNING)
-            Assert(0);
+            ECCODES_ASSERT(0);
     }
 }
 
@@ -237,10 +237,21 @@ static void default_print(const grib_context* c, void* descriptor, const char* m
 void grib_context_set_print_proc(grib_context* c, grib_print_proc p)
 {
     c = c ? c : grib_context_get_default();
-    /* Set logging back to the default if p is NULL */
+    /* Set printing back to the default if p is NULL */
     c->print = (p ? p : &default_print);
 }
 
+void grib_context_set_data_quality_checks(grib_context* c, int val)
+{
+    c = c ? c : grib_context_get_default();
+    // If val == 0, disable data quality checks
+    // If val == 1, failure results in an error
+    // If val == 2, failure results in a warning
+    ECCODES_ASSERT(val == 0 || val == 1 || val == 2);
+
+    c->grib_data_quality_checks = val;
+}
+
 void grib_context_set_debug(grib_context* c, int mode)
 {
     c = c ? c : grib_context_get_default();
@@ -287,6 +298,7 @@ static grib_context default_grib_context = {
     0,               /* no_big_group_split         */
     0,               /* no_spd                     */
     0,               /* keep_matrix                */
+    0,               /* show_hour_stepunit         */
     0,               /* grib_definition_files_path */
     0,               /* grib_samples_path          */
     0,               /* grib_concept_path          */
@@ -357,6 +369,7 @@ static grib_context default_grib_context = {
     0,              /* bufr_multi_element_constant_arrays */
     0,              /* grib_data_quality_checks   */
     0,              /* single_precision           */
+    0,              /* eckit_geo             */
     0,              /* log_stream                 */
     0,              /* classes                    */
     0,              /* lists                      */
@@ -373,7 +386,7 @@ static grib_context default_grib_context = {
 
 grib_context* grib_context_get_default()
 {
-    GRIB_MUTEX_INIT_ONCE(&once, &init);
+    GRIB_MUTEX_INIT_ONCE(&once, &init_mutex);
     GRIB_MUTEX_LOCK(&mutex_c);
 
     if (!default_grib_context.inited) {
@@ -388,23 +401,28 @@ grib_context* grib_context_get_default()
         const char* no_big_group_split                  = NULL;
         const char* no_spd                              = NULL;
         const char* keep_matrix                         = NULL;
+        const char* show_hour_stepunit                  = NULL;
         const char* bufrdc_mode                         = NULL;
         const char* bufr_set_to_missing_if_out_of_range = NULL;
         const char* bufr_multi_element_constant_arrays  = NULL;
         const char* grib_data_quality_checks            = NULL;
         const char* single_precision                    = NULL;
+        const char* eckit_geo                           = NULL;
         const char* file_pool_max_opened_files          = NULL;
 
 #ifdef ENABLE_FLOATING_POINT_EXCEPTIONS
         feenableexcept(FE_ALL_EXCEPT & ~FE_INEXACT);
 #endif
 
-        write_on_fail                       = codes_getenv("ECCODES_GRIB_WRITE_ON_FAIL");
         bufrdc_mode                         = getenv("ECCODES_BUFRDC_MODE_ON");
         bufr_set_to_missing_if_out_of_range = getenv("ECCODES_BUFR_SET_TO_MISSING_IF_OUT_OF_RANGE");
         bufr_multi_element_constant_arrays  = getenv("ECCODES_BUFR_MULTI_ELEMENT_CONSTANT_ARRAYS");
         grib_data_quality_checks            = getenv("ECCODES_GRIB_DATA_QUALITY_CHECKS");
         single_precision                    = getenv("ECCODES_SINGLE_PRECISION");
+        file_pool_max_opened_files          = getenv("ECCODES_FILE_POOL_MAX_OPENED_FILES");
+        eckit_geo                           = getenv("ECCODES_ECKIT_GEO");
+        // The following had an equivalent env. var in grib_api
+        write_on_fail                       = codes_getenv("ECCODES_GRIB_WRITE_ON_FAIL");
         large_constant_fields               = codes_getenv("ECCODES_GRIB_LARGE_CONSTANT_FIELDS");
         no_abort                            = codes_getenv("ECCODES_NO_ABORT");
         debug                               = codes_getenv("ECCODES_DEBUG");
@@ -415,7 +433,7 @@ grib_context* grib_context_get_default()
         no_big_group_split                  = codes_getenv("ECCODES_GRIB_NO_BIG_GROUP_SPLIT");
         no_spd                              = codes_getenv("ECCODES_GRIB_NO_SPD");
         keep_matrix                         = codes_getenv("ECCODES_GRIB_KEEP_MATRIX");
-        file_pool_max_opened_files          = getenv("ECCODES_FILE_POOL_MAX_OPENED_FILES");
+        show_hour_stepunit                  = codes_getenv("ECCODES_GRIB_HOURLY_STEPS_WITH_UNITS");
 
         /* On UNIX, when we read from a file we get exactly what is in the file on disk.
          * But on Windows a file can be opened in binary or text mode. In binary mode the system behaves exactly as in UNIX.
@@ -429,6 +447,7 @@ grib_context* grib_context_get_default()
         default_grib_context.no_big_group_split = no_big_group_split ? atoi(no_big_group_split) : 0;
         default_grib_context.no_spd = no_spd ? atoi(no_spd) : 0;
         default_grib_context.keep_matrix = keep_matrix ? atoi(keep_matrix) : 1;
+        default_grib_context.grib_hourly_steps_with_units = show_hour_stepunit ? atoi(show_hour_stepunit) : 0;
         default_grib_context.write_on_fail = write_on_fail ? atoi(write_on_fail) : 0;
         default_grib_context.no_abort = no_abort ? atoi(no_abort) : 0;
         default_grib_context.debug = debug ? atoi(debug) : 0;
@@ -449,7 +468,7 @@ grib_context* grib_context_get_default()
 
 #ifdef ECCODES_SAMPLES_PATH
         if (!default_grib_context.grib_samples_path)
-            default_grib_context.grib_samples_path = ECCODES_SAMPLES_PATH;
+            default_grib_context.grib_samples_path = (char*)ECCODES_SAMPLES_PATH;
 #endif
 
         default_grib_context.grib_definition_files_path = codes_getenv("ECCODES_DEFINITION_PATH");
@@ -531,11 +550,11 @@ grib_context* grib_context_get_default()
         }
 #endif
 
-        grib_context_log(&default_grib_context, GRIB_LOG_DEBUG, "Definitions path: %s",
-                         default_grib_context.grib_definition_files_path);
-        grib_context_log(&default_grib_context, GRIB_LOG_DEBUG, "Samples path:     %s",
-                         default_grib_context.grib_samples_path);
-
+        if (default_grib_context.debug) {
+            fprintf(stderr, "ECCODES DEBUG ecCodes Version:  %s\n", ECCODES_VERSION_STR);
+            fprintf(stderr, "ECCODES DEBUG Definitions path: %s\n", default_grib_context.grib_definition_files_path);
+            fprintf(stderr, "ECCODES DEBUG Samples path:     %s\n", default_grib_context.grib_samples_path);
+        }
         default_grib_context.keys_count = 0;
         default_grib_context.keys       = grib_hash_keys_new(&(default_grib_context), &(default_grib_context.keys_count));
 
@@ -549,6 +568,7 @@ grib_context* grib_context_get_default()
         default_grib_context.bufr_multi_element_constant_arrays = bufr_multi_element_constant_arrays ? atoi(bufr_multi_element_constant_arrays) : 0;
         default_grib_context.grib_data_quality_checks = grib_data_quality_checks ? atoi(grib_data_quality_checks) : 0;
         default_grib_context.single_precision = single_precision ? atoi(single_precision) : 0;
+        default_grib_context.eckit_geo = eckit_geo ? atoi(eckit_geo) : 0;
         default_grib_context.file_pool_max_opened_files = file_pool_max_opened_files ? atoi(file_pool_max_opened_files) : DEFAULT_FILE_POOL_MAX_OPENED_FILES;
     }
 
@@ -566,7 +586,7 @@ grib_context* grib_context_get_default()
 
 //     if (!parent) parent=grib_context_get_default();
 
-//     GRIB_MUTEX_INIT_ONCE(&once,&init);
+//     GRIB_MUTEX_INIT_ONCE(&once,&init_mutex);
 //     GRIB_MUTEX_LOCK(&(parent->mutex));
 
 //     c = (grib_context*)grib_context_malloc_clear_persistent(&default_grib_context,sizeof(grib_context));
@@ -643,7 +663,7 @@ static int init_definition_files_dir(grib_context* c)
     strncpy(path, c->grib_definition_files_path, ECC_PATH_MAXLEN-1);
     path[ ECC_PATH_MAXLEN - 1 ] = '\0';
 
-    GRIB_MUTEX_INIT_ONCE(&once, &init);
+    GRIB_MUTEX_INIT_ONCE(&once, &init_mutex);
     GRIB_MUTEX_LOCK(&mutex_c);
 
     p = path;
@@ -658,20 +678,19 @@ static int init_definition_files_dir(grib_context* c)
     }
     else {
         /* Definitions path contains multiple directories */
-        char* dir = NULL;
-        dir       = strtok_r(path, ECC_PATH_DELIMITER_STR, &lasts);
+        const char* dir = strtok_r(path, ECC_PATH_DELIMITER_STR, &lasts);
 
         while (dir != NULL) {
             if (next) {
                 next->next = (grib_string_list*)grib_context_malloc_clear_persistent(c, sizeof(grib_string_list));
-                next       = next->next;
+                next = next->next;
             }
             else {
                 c->grib_definition_files_dir = (grib_string_list*)grib_context_malloc_clear_persistent(c, sizeof(grib_string_list));
-                next                         = c->grib_definition_files_dir;
+                next = c->grib_definition_files_dir;
             }
             next->value = codes_resolve_path(c, dir);
-            dir         = strtok_r(NULL, ECC_PATH_DELIMITER_STR, &lasts);
+            dir = strtok_r(NULL, ECC_PATH_DELIMITER_STR, &lasts);
         }
     }
 
@@ -689,7 +708,7 @@ char* grib_context_full_defs_path(grib_context* c, const char* basename)
     if (!c)
         c = grib_context_get_default();
 
-    GRIB_MUTEX_INIT_ONCE(&once, &init);
+    GRIB_MUTEX_INIT_ONCE(&once, &init_mutex);
 
     if (*basename == '/' || *basename == '.') {
         return (char*)basename;
@@ -717,7 +736,7 @@ char* grib_context_full_defs_path(grib_context* c, const char* basename)
             snprintf(full, sizeof(full), "%s/%s", dir->value, basename);
             if (codes_access(full, F_OK) == 0) { /* 0 means file exists */
                 fullpath = (grib_string_list*)grib_context_malloc_clear_persistent(c, sizeof(grib_string_list));
-                Assert(fullpath);
+                ECCODES_ASSERT(fullpath);
                 fullpath->value = grib_context_strdup(c, full);
                 GRIB_MUTEX_LOCK(&mutex_c);
                 grib_trie_insert(c->def_files, basename, fullpath);
@@ -786,8 +805,8 @@ void grib_context_reset(grib_context* c)
 
             a = fr->root;
             while (a) {
-                grib_action* na = a->next;
-                grib_action_delete(c, a);
+                grib_action* na = a->next_;
+                delete a;
                 a = na;
             }
             grib_context_free_persistent(c, fr->filename);
@@ -808,14 +827,14 @@ void grib_context_reset(grib_context* c)
 
     if (c->grib_definition_files_dir) {
         grib_string_list* next = c->grib_definition_files_dir;
-        grib_string_list* cur  = NULL;
+        grib_string_list* cur = NULL;
         while (next) {
             cur  = next;
             next = next->next;
             grib_context_free(c, cur->value);
             grib_context_free(c, cur);
         }
-        c->grib_definition_files_dir=0;
+        c->grib_definition_files_dir = 0;
     }
 
     if (c->multi_support_on)
@@ -836,7 +855,6 @@ void grib_context_reset(grib_context* c)
 
 void grib_context_delete(grib_context* c)
 {
-    size_t i = 0;
     if (!c)
         c = grib_context_get_default();
 
@@ -848,12 +866,12 @@ void grib_context_delete(grib_context* c)
     if (c != &default_grib_context)
         grib_context_free_persistent(&default_grib_context, c);
 
-    for(i=0; i<MAX_NUM_HASH_ARRAY; ++i)
+    for(size_t i=0; i<MAX_NUM_HASH_ARRAY; ++i)
         c->hash_array[i] = NULL;
     c->hash_array_count = 0;
     grib_itrie_delete(c->hash_array_index);
     c->hash_array_index=0;
-    grib_trie_delete(c->expanded_descriptors);
+    grib_trie_delete_container(c->expanded_descriptors);
     c->expanded_descriptors=0;
 
     c->inited = 0;
@@ -878,7 +896,7 @@ void grib_context_set_definitions_path(grib_context* c, const char* path)
 {
     if (!c)
         c = grib_context_get_default();
-    GRIB_MUTEX_INIT_ONCE(&once, &init);
+    GRIB_MUTEX_INIT_ONCE(&once, &init_mutex);
     GRIB_MUTEX_LOCK(&mutex_c);
 
     c->grib_definition_files_path = strdup(path);
@@ -890,7 +908,7 @@ void grib_context_set_samples_path(grib_context* c, const char* path)
 {
     if (!c)
         c = grib_context_get_default();
-    GRIB_MUTEX_INIT_ONCE(&once, &init);
+    GRIB_MUTEX_INIT_ONCE(&once, &init_mutex);
     GRIB_MUTEX_LOCK(&mutex_c);
 
     c->grib_samples_path = strdup(path);
@@ -1013,26 +1031,6 @@ void* grib_context_buffer_malloc_clear(const grib_context* c, size_t size)
     return p;
 }
 
-void grib_context_set_memory_proc(grib_context* c, grib_malloc_proc m, grib_free_proc f, grib_realloc_proc r)
-{
-    c->free_mem    = f;
-    c->alloc_mem   = m;
-    c->realloc_mem = r;
-}
-
-void grib_context_set_persistent_memory_proc(grib_context* c, grib_malloc_proc m, grib_free_proc f)
-{
-    c->free_persistent_mem  = f;
-    c->alloc_persistent_mem = m;
-}
-
-void grib_context_set_buffer_memory_proc(grib_context* c, grib_malloc_proc m, grib_free_proc f, grib_realloc_proc r)
-{
-    c->free_buffer_mem    = f;
-    c->alloc_buffer_mem   = m;
-    c->realloc_buffer_mem = r;
-}
-
 void grib_context_set_data_accessing_proc(grib_context* c, grib_data_read_proc read, grib_data_write_proc write, grib_data_tell_proc tell)
 {
     c->read  = read;
@@ -1093,7 +1091,7 @@ int grib_context_get_handle_file_count(grib_context* c)
     int r = 0;
     if (!c)
         c = grib_context_get_default();
-    GRIB_MUTEX_INIT_ONCE(&once, &init);
+    GRIB_MUTEX_INIT_ONCE(&once, &init_mutex);
     GRIB_MUTEX_LOCK(&mutex_c);
     r = c->handle_file_count;
     GRIB_MUTEX_UNLOCK(&mutex_c);
@@ -1104,7 +1102,7 @@ int grib_context_get_handle_total_count(grib_context* c)
     int r = 0;
     if (!c)
         c = grib_context_get_default();
-    GRIB_MUTEX_INIT_ONCE(&once, &init);
+    GRIB_MUTEX_INIT_ONCE(&once, &init_mutex);
     GRIB_MUTEX_LOCK(&mutex_c);
     r = c->handle_total_count;
     GRIB_MUTEX_UNLOCK(&mutex_c);
@@ -1115,7 +1113,7 @@ void grib_context_set_handle_file_count(grib_context* c, int new_count)
 {
     if (!c)
         c = grib_context_get_default();
-    GRIB_MUTEX_INIT_ONCE(&once, &init);
+    GRIB_MUTEX_INIT_ONCE(&once, &init_mutex);
     GRIB_MUTEX_LOCK(&mutex_c);
     c->handle_file_count = new_count;
     GRIB_MUTEX_UNLOCK(&mutex_c);
@@ -1124,7 +1122,7 @@ void grib_context_set_handle_total_count(grib_context* c, int new_count)
 {
     if (!c)
         c = grib_context_get_default();
-    GRIB_MUTEX_INIT_ONCE(&once, &init);
+    GRIB_MUTEX_INIT_ONCE(&once, &init_mutex);
     GRIB_MUTEX_LOCK(&mutex_c);
     c->handle_total_count = new_count;
     GRIB_MUTEX_UNLOCK(&mutex_c);
@@ -1134,7 +1132,7 @@ void grib_context_increment_handle_file_count(grib_context* c)
 {
     if (!c)
         c = grib_context_get_default();
-    GRIB_MUTEX_INIT_ONCE(&once, &init);
+    GRIB_MUTEX_INIT_ONCE(&once, &init_mutex);
     GRIB_MUTEX_LOCK(&mutex_c);
     c->handle_file_count++;
     GRIB_MUTEX_UNLOCK(&mutex_c);
@@ -1143,7 +1141,7 @@ void grib_context_increment_handle_total_count(grib_context* c)
 {
     if (!c)
         c = grib_context_get_default();
-    GRIB_MUTEX_INIT_ONCE(&once, &init);
+    GRIB_MUTEX_INIT_ONCE(&once, &init_mutex);
     GRIB_MUTEX_LOCK(&mutex_c);
     c->handle_total_count++;
     GRIB_MUTEX_UNLOCK(&mutex_c);
@@ -1158,7 +1156,7 @@ bufr_descriptors_array* grib_context_expanded_descriptors_list_get(grib_context*
     if (!c)
         c = grib_context_get_default();
 
-    GRIB_MUTEX_INIT_ONCE(&once, &init);
+    GRIB_MUTEX_INIT_ONCE(&once, &init_mutex);
     GRIB_MUTEX_LOCK(&mutex_c);
 
     if (!c->expanded_descriptors) {
@@ -1198,7 +1196,7 @@ void grib_context_expanded_descriptors_list_push(grib_context* c,
     if (!c)
         c = grib_context_get_default();
 
-    GRIB_MUTEX_INIT_ONCE(&once, &init);
+    GRIB_MUTEX_INIT_ONCE(&once, &init_mutex);
     GRIB_MUTEX_LOCK(&mutex_c);
 
     newdescriptorsList             = (bufr_descriptors_map_list*)grib_context_malloc_clear(c, sizeof(bufr_descriptors_map_list));
@@ -1231,7 +1229,7 @@ void codes_assertion_failed(const char* message, const char* file, int line)
     /* Default behaviour is to abort
      * unless user has supplied his own assertion routine */
     if (assertion == NULL) {
-        grib_context* c = grib_context_get_default();
+        const grib_context* c = grib_context_get_default();
         fprintf(stderr, "ecCodes assertion failed: `%s' in %s:%d\n", message, file, line);
         if (!c->no_abort) {
             abort();
@@ -1244,7 +1242,7 @@ void codes_assertion_failed(const char* message, const char* file, int line)
     }
 }
 
-int grib_get_gribex_mode(grib_context* c)
+int grib_get_gribex_mode(const grib_context* c)
 {
     if (!c)
         c = grib_context_get_default();
